package com.jkcredit.asychronous.util;

import com.jcraft.jsch.ChannelSftp;
import com.jcraft.jsch.JSch;
import com.jcraft.jsch.Session;
import lombok.extern.slf4j.Slf4j;

import java.io.*;
import java.text.SimpleDateFormat;
import java.util.Properties;

/**
 * @description:
 * @author: xusonglin
 * @create: 2020/2/3 17:19
 * @version: V1.0
 **/
@Slf4j
public class SFTPUtil {
    static JSch jsch = null;
    static Session session = null;
    static ChannelSftp channel = null;

    /**
     * 密钥连接到指定的IP
     *
     * @param keyFilePath 密钥路径
     * @param passphrase  密钥的密码
     */
    public static void connect(String userName, String host, int port, String passphrase, String keyFilePath) throws Exception {
        try {
            jsch = new JSch();
            if (keyFilePath != null) {
                if (passphrase != null) {
                    jsch.addIdentity(keyFilePath, passphrase);// 设置私钥
                } else {
                    jsch.addIdentity(keyFilePath);// 设置私钥
                }
                log.info("连接sftp，私钥文件路径：" + keyFilePath);
            }
            log.info("SFTP Host: " + host + "; UserName:" + userName);
            session = jsch.getSession(userName, host, port);
            log.debug("Session 已建立.");
            Properties sshConfig = new Properties();
            sshConfig.put("StrictHostKeyChecking", "no");
            session.setConfig(sshConfig);
            session.setConfig("kex", "diffie-hellman-group1-sha1");
            session.connect();
            log.debug("Session 已连接.");
            channel = (ChannelSftp) session.openChannel("sftp");
            channel.connect(60000);
            log.info("连接到SFTP成功.Host: " + host);
        } catch (Exception e) {
            log.error("连接SFTP失败：", e);
        }
    }

    /**
     * 关闭连接
     */
    private static void close() {
        if (channel != null) {
            try {
                channel.disconnect();
            } catch (Exception e) {
                log.error("sftp close channel failed...", e);
            }
        }
        if (session != null) {
            try {
                session.disconnect();
            } catch (Exception e) {
                log.error("sftp close session failed...", e);
            }
        }
    }

    public static void upload(String host, Integer port, String userName, String passphrase, String dir,String fileName, String keyFilePath) throws Exception {
        //连接sftp
        connect(userName, host, port, passphrase, keyFilePath);
        if (channel == null) {
            log.info("SFTP服务器未连接");
            return;
        }
        try {
//            fileName = "C:\\Users\\Administrator\\Desktop\\ftp\\upload\\1580697240002.xlsx";
//            channel.put(new FileInputStream(fileName), "/download/1580697240002.xlsx", ChannelSftp.OVERWRITE);
            log.info("dis == " + dir);
            channel.put(new FileInputStream(fileName), dir, ChannelSftp.OVERWRITE);
            log.info("上传到sftp成功");
        } catch (Exception e) {
            log.info("上传文件：" + fileName + "失败");
            throw e;
        } finally {
            close();
        }

        // todo 秘钥文件路径有问题 需要测试上传  下载需要在util中写个方法，替换ftp文件夹下的route
    }


}
